@@ -459,6 +459,18 @@ except socket.error:
 # SERVICE-SPECIFIC CONFIGS BELOW
 # -----
 
+# port ranges for external service instances (f.e. elasticsearch clusters, opensearch clusters,...)
+EXTERNAL_SERVICE_PORTS_START = int(
+    os.environ.get("EXTERNAL_SERVICE_PORTS_START")
+    or os.environ.get("SERVICE_INSTANCES_PORTS_START")
+    or 4510
+)
+EXTERNAL_SERVICE_PORTS_END = int(
+    os.environ.get("EXTERNAL_SERVICE_PORTS_END")
+    or os.environ.get("SERVICE_INSTANCES_PORTS_END")
+    or (EXTERNAL_SERVICE_PORTS_START + 30)
+)
+
 # java options to Lambda
 LAMBDA_JAVA_OPTS = os.environ.get("LAMBDA_JAVA_OPTS", "").strip()
 
@@ -793,9 +805,10 @@ def get_protocol():
     return "https" if USE_SSL else "http"
 
 
-def external_service_url(service_key, host=None):
+def external_service_url(service_key, host=None, port=None):
     host = host or HOSTNAME_EXTERNAL
-    return "%s://%s:%s" % (get_protocol(), host, service_port(service_key))
+    port = port or service_port(service_key)
+    return "%s://%s:%s" % (get_protocol(), host, port)
 
 
 def get_edge_port_http():
